/*
** fury.h - the wonky protocol used over the modem
**
** Keith Zaback (krz@wcug.wwu.edu) Apr 98
*/

#ifndef INC_FURY_H
#define INC_FURY_H

#include "furytypes.h"
#include "defaults.h"
#include "hdrcache.h"
#include "buffer.h"
#include "hash.h"

/* 2 byte header + 3 hdr cache info + 1500 data (ethernet max) */
#define FURY_FRAME_MAX			1505
#define FURY_FRAME_MAX_DATA		1500
#define FURY_IPX_FRAME_MAX		1500
#define FURY_ETHER_FRAME_MAX		1514


/* diff problems */
#define FURY_DIFF_RESET_START		8

/* caching & diff info */
typedef struct FURY_CACHE_PTRS {
	HDRCACHE		*send_hdrcache;	/* outgoing pkt cache */
	HDRCACHE		*recv_hdrcache;	/* incoming pkt cache */
	HASHTABLE		*send_pktcache;	/* outgoing pkt cache */
	HASHTABLE		*recv_pktcache;	/* incoming pkt cache */
	
	BYTE			*last_senddata;
	int			*last_sendsize;
	BYTE			*last_recvdata;
	int			*last_recvsize;
	int			*diff_reset;	/* a counter so we don't go insane */
} FURY_CACHE_PTRS;

/* sent in the place of an ipx frame header */
typedef struct stHDRCACHE_INFO {
	BYTE		index;		/* where in the cache */
	unsigned short	length __attribute__((packed));  /* length to put in the ipx frame */
} HDRCACHE_INFO;


/* more lcp stuff */
#define FLCP_PACKET_MAX		128
#define FLCP_DATA_MAX		(FLCP_PACKET_MAX-4)

typedef struct stFLCP_PACKETHEADER {
	BYTE		code;
	BYTE		id;
	unsigned short	length __attribute__ ((packed));
} FLCP_PACKETHEADER;

/* types of lcp packets we like */
#define FLCP_CONFIG_REQ		1
#define FLCP_CONFIG_ACK		2
#define FLCP_CONFIG_NAK		3
#define FLCP_CONFIG_REJ		4
#define FLCP_TERMINATE_REQ	5
#define FLCP_TERMINATE_ACK	6
#define FLCP_ECHO_REQ		7
#define FLCP_ECHO_REPLY		8

/* max restart values */
#define FURY_TIMEOUT			2	/* 2 seconds */
#define FURY_RESTART_CONFIGURE_MAX	5
#define FURY_RESTART_TERMINATE_MAX	5

/* this is for the status of the connection */
#define FURY_STATE_INITIAL	0
#define FURY_STATE_STARTING	1
#define FURY_STATE_CLOSED	2
#define FURY_STATE_STOPPED	3
#define FURY_STATE_CLOSING	4
#define FURY_STATE_STOPPING	5
#define FURY_STATE_REQ_SENT	6
#define FURY_STATE_ACK_RCVD	7
#define FURY_STATE_ACK_SENT	8
#define	FURY_STATE_OPENED	9

/* various events */
#define FURY_EVENT_UP		0
#define FURY_EVENT_DOWN		1
#define FURY_EVENT_OPEN		2
#define FURY_EVENT_CLOSE	3
#define FURY_EVENT_TON		4	/* timeout non-zero counter */
#define FURY_EVENT_TOZ		5	/* timeout zero-counter */
#define FURY_EVENT_RCR		6
#define FURY_EVENT_RCR_BAD	7
#define FURY_EVENT_RCA		8
#define FURY_EVENT_RCN		9
#define FURY_EVENT_RTR		10
#define FURY_EVENT_RTA		11

#define FHDR_LENGTH(x)		((x & 0x07ff))
#define FHDR_TYPE(x)		(((x & 0x3800) >> 11))
#define FHDR_CACHED_DATA(x)	(x & 0x8000)
#define FHDR_DIFF(x)		(x & 0x4000)

#define FTYPE_DIFF		0x4000
#define FTYPE_PKT_CACHED	0x8000

/* fury packet types (3 bits = 8 types) */
#define FTYPE_LCP		0	/* link control protocol */
#define FTYPE_IPX		1	/* ipx data comming! */
#define FTYPE_ACK		2	/* for ack'ing things */
#define FTYPE_COMMAND		3	/* other complex commands */

/* ack types */
#define FACK_RESET		1	/* reset ACK */

#define ACK_MAX_SIZE		16

/* command types */
#define FCMD_CLEAR_HDRCACHE	0	/* request for clearing a cache line */
#define FCMD_CLEAR_PKTCACHE	1	/* request for pkt cache clearing */
#define FCMD_RESET		2	/* request total reset */
#define FCMD_DIFF_RESET		3	/* stop using diffs! */

#define CMD_MAX_SIZE		64

/* flags used to encode the data for xfer */
#define FURY_FLAG		0x7d	/* marker of start/end of frame */
#define FURY_ESCAPE		0x7e	/* for byte stuffing */
#define FURY_INVERT		0x20	/* xor with this when byte stuffed */

typedef struct stFURYFRAME {
	int		count;		/* how many bytes in the data[] block */
	BYTE		escape;		/* value to xor with input bytes */
	BYTE		drop;		/* if true, ignore frame */
	BYTE		data[FURY_FRAME_MAX*2];	/* to account for stuffing */
} FURYFRAME;

#endif
