/*
** starcraft.c - starcraft optimizations
**
** Keith Zaback (krz@wcug.wwu.edu) Oct 98
**
*/

/* includes */
#include "ipxfury.h"

/* options */
#define SHOW_SENDS	0
#define SHOW_DROPS	0

/* how many resends in a row until we actually send one */
#define FURY_SC_RESEND_MAX		20

/* globals */
extern GLOBALS	ifg;

/*
**	STARCRAFT_FILTER -
*/
int starcraft_filter(INTERFACE *edev, BYTE *data, HWADDR_ENTRY *src)
{
	IPXHEADER	*ipxheader;
	int		index, type;
	
	/* set ptr */
	ipxheader = (IPXHEADER *)data;
	
	/* proper ports? */
	if( ntohs(ipxheader->ipx_dest.port) == 6112 ) {
		/* type 0x02 seems to be game data.. */
		type = data[30+8];
		
		if( type == 0x02 ) {
			/* 
			** Now.. due to modem lag and such, we aren't
			** as responsive as a LAN, but the data is
			** flowing.. so these packets are not needed
			** _UNLESS_ we have sent a bunch of them
			** Starcraft sends about 20 of these a second..
			** when it is "waiting for players"
			*/
			
			/* get the index (player number: offset 10 (i hope)) */
			index = data[30+10];
			if( (index < 0) || (index > 7) ) {
				/* info! */
				notice("[sc] player number is loony! (%d)\n", index);
				index = 0;
			}
			
			/* is this a resend/resync packet?			*/
			/* I'm not sure if this is always 0x02.. I have only	*/
			/* been able to test with 2 player games.. 		*/
			/* well i did a 3 player test (10/19) and it was still	*/
			/* 0x02							*/
			if( data[30+11] == 0x02 ) {
			
				/* have we reached the max? */
				if( src->sc_resend_counter[index] > FURY_SC_RESEND_MAX ) {
					/* restart counter.. and use this packet */
					src->sc_resend_counter[index] = 0x00;
#if SHOW_SENDS == 1			
					notice("[sc] Sending Starcraft resend packet\n");
#endif
					/* use this */
					return(1);
				}
			
				/* else -- add to the counter */
				src->sc_resend_counter[index]++;

				/* do some logging */
#if SHOW_DROPS == 1
				fprintf(ifg.logfile, "Starcraft Filter: DROPPED\n");
				fhexdump(ifg.logfile, &data[30], ntohs(ipxheader->length) - 30);
#endif				
			
				/* drop */
				return(0);
			} else {
				/* this is game data.. reset counter */
				src->sc_resend_counter[index] = 0x00;
			}
		}				
	}
	
	/* use this -- not interesting */
	return(1);

}



